function vballgprb(g, h, alltran, basisf, subsets, refsubs, refsubspos, subspriocnt, gamma, itermax, acc_eps)
% function [vbprb, g_vals]=vballgprb(a, b, g, alltran, basisf, subsets, refsubs, refsubspos, subsprio, gamma, itermax, acc_eps)
% constructs a vbprb object that represents a hierarchical graphical model for
% calculating the posterior probability of two indicator variables: one for the
% subset model, the other for the reference subset.
%
% In:
% g, h:     Gamma prior over lambda (the latter is the precision of the Gaussian prior over 
%           regression models). g and h are the shape and scale parameters of a gamma distribution over
%           the individual prior precisions (i.e. lambda mentioned above). This
%           hierarchical prior allows for less sensitive Gaussian priors over regression parameters.
% alltran : training data. In this modified version tran is actuallly a cell array, each entry representing a system that
%           provides information about the same indicator variables. Each entry is of originla tran struture thus with 
%           columns representing an input dimension. The last column determines the 
%           dichotomous classification of the row (i.e. a (0, 1) or (1, 2) value).
% basisf:   description of the basis function to be applied to each subset before regression.
%           basisf.btype:   type of basis functions ('g' for Gaussian and 
%                           't' for thin plate spline, 'l' for none)
%           basisf.prec:    precision of basis function used by 'g'.
%           basisf.nr:      number of basis functions
% subsets:  a cell array of column indices into tran (the last column would not make sense!)
%           selects the regressoprs to be used by the corresponding GLM.
% refsubs:  a reference subset used to compare against each subset of the above.
% refsubspos: position of the reference subset in the original subset list.
% subspriocnt: prior counts of the common prior probability over the subset indicators. [1 x 2] values with
%           subspriocnt(1) denoting the counts in favour of the reference subset and subspriocnt(2) in 
%           favour of the individual subsets.
% gamma:    precision of the latent variable Gaussian cdf we use as (inverse) GLM probit link.
% itermax:  maximal number of iterations of the optimization algorithm.
% acc_eps:  maximal absolute difference between two sucessive approximate log evidence before
%           the optimization terminates.
%
% Out:
% vbprb:    The object that represents the DAG.
% g_vals:   various internal quantities required for updating the graph.
%           modification: g_vals is an array with nsys entries each representing 
%           the situation of one system.
%           g_vals(n).g_in:    cell array of all [nsubsets x 2] input vectors that 
%                           correspond to the subset comparison.
%                       g_in{n, 1} corresponds to Ig==0
%                       g_in{n, 2} corresponds to Ig==1
%           g_vals(n).an,  
%           g_vals(n).bn:  all latent cutoffs as implied by the targets. (codes all P(y|z)) 
%                       feasible range for each zn in the open interval [an, bn]
%           g_vals(n).trg: direct target representation used in some functions.
%
% Definition of the vbprb:
%
% common values:
% vbprb.i_Priocnt:  [1 x 2] stores the prior counts for "reference subsets" versus individual subsets.
% vbprb.i_Postcnt:  [1 x 2] posterior counts of the beta distribution over
%                   the prior indicator probabilities.
%                   Both together specify a hierarchical prior over model
%                   indicators, which allows for some adjusting to each
%                   problem (or set of problems in the planned expension).
% vbprb.gamma:      stores gamma (probit link precision)
% vbprb.g,
% vbprb.h:          define the gamma prior over lambda 
% vbprb.allIgprobs: Approximate probabilities P(Ig|D,lambda) for all subsets and indicator levels
%                   a [2 x nsubsets] matrix where (1,:) represents the probabilities for Ig==0
%                   and (2,:) the probabilities for Ig==1.
% vbprb.lgevids:    [niter x 1] approximate log evidences of the graphical model. 
%                   (for debugging and monitoring convergence)
% vbprb.acceps:     terminal accuracy (i.e. maximal absolute difference between succesive
%                   approximate log evidences before optimization terminates.
% vbprb.maxit:      maximal number of iterations in the optimization.
% vbprb.basisf:     functional description of basis functions.
%                   basisf.btype:   type of basis functions ('g' for Gaussian and 
%                                   't' for thin plate spline, 'l' for none)
%                   basisf.prec:    precision of basis function used by 'g'.
%                   basisf.nr:      number of basis functions
% system specific vlues:
% vbprb.sys(n)
% sys.g_p, 
% sys.h_p:          define the gamma posterior over lambdas's i.e. [maxdim x 1]
% sys.g_mdls:       approximate parameter distributions of the [nsubsets x 1] regression models
% g_mdl.i_mdl:      [2 x 1] i_mdl(1): parameters for Ig==0 i.e. for the reference subset,
%                   i_mdl(2): parameters for Ig==1 i.e. for the corresponding subset.                    
% i_mdl.beta:       MAP regressor (i.e. mean of the Gaussian)
% i_mdl.P:          precision matrix of the Gaussian
% i_mdl.C:          covariance matrix (i.e. inverse of the precision).
% i_mdl.K:          nonlinear projection kernel (used by nonlinear basis functions).
% i_mdl.subs:       corresponding subset
% sys.latmeans:     means of the latent variables of all regression models (i.e. one per 
%                   subset and indicator state).
%                   [nsamples x nsubsets x 2] - (:,:,1) - for all Ig==0
%                                             - (:,:,2) - for all Ig==1
%                   together with the global cutoffs and precision vbprb.gamma
%                   this fully specifies all latent distributions.
% sys.alldim:       [nsubsets x 2] - dimensions of all input vectors.
%
% (C) P. Sykacek 2005

global vbprb;
global g_vals;

% some initial initialisations
% length(alltran)
% size(alltran{1})
% size(alltran{2})
vbprb.refsubspos=refsubspos;
vbprb.maxit=itermax;
vbprb.acceps=acc_eps;
vbprb.gamma=gamma;
vbprb.g=g;
vbprb.h=h;
% column vectors!
vbprb.i_Priocnt=subspriocnt(:);
vbprb.i_Postcnt=subspriocnt(:);
vbprb.basisf=basisf;
nsys=length(alltran);
for sysno=1:nsys
    % extract target data:
    tran=alltran{sysno};
    % now convert targets into feasibility range
    ntcols=size(tran,2);
    trg=tran(:,ntcols);
    if max(trg)==2
        trg=trg-1;
        tran(:, ntcols)=trg;
    end
    % generate the truncation limits for the distributions
    % note: the Gaussian is "on" between an and bn.
    nsmpl=size(trg,1);
    an=zeros(nsmpl,1);
    bn=an;
    bn(find(trg==1))=inf;
    an(find(trg==0))=-inf;
    g_vals(sysno).an=an;
    g_vals(sysno).bn=bn;
    g_vals(sysno).trg=trg;
    % now we initialize the starting point of the approximate distributions
    % such that we are close to the optimum and need only a few iterations 
    % in the vb run.
    nsubsets=length(subsets);
    allz=zeros(nsmpl, nsubsets, 2);
    alldim=zeros(nsubsets, 2);
    n=1;
    maxparlen=0;
    % get the expected prior precision:
    explambda=g/h;
    while n <= nsubsets
        subset=subsets{n};
        g_mdls(n).i_mdl(2).subs=subset;
        [in, K] = getinputs(basisf, subset, tran);
        n_in=size(in,2);
        alldim(n,2)=n_in;
        if maxparlen < n_in
            maxparlen=n_in;
        end
        g_in{n, 2}=in;
        g_mdls(n).i_mdl(2).K=K;
        % get ml regressor and latent variables (call probitem with weight decay penalty).
        [wn, mzn]=probitem(in, trg, vbprb.gamma, vbprb.acceps, vbprb.maxit, diag(explambda(ones(1, n_in))));
        g_mdls(n).i_mdl(2).beta=wn(:);
        g_mdls(n).i_mdl(2).P=diag(explambda(ones(1, n_in)));
        g_mdls(n).i_mdl(2).C=diag(1./explambda(ones(1, n_in)));
        g_mdls(n).i_mdl(2).n_in=n_in;
        % store latent ml values
        allz(:,n,2)=mzn(:);
        % now the same for the reference subset
        subset=refsubs;
        g_mdls(n).i_mdl(1).subs=subset;
        [in, K] = getinputs(basisf, subset, tran);
        n_in=size(in,2);
        alldim(n,1)=n_in;    
        if maxparlen < n_in
            maxparlen=n_in;
        end
        % as extension to the previous implementation we do know add
        % the posterior paraeters of the prior over lambda for this subset:
        % this_dim=max(alldim(n,:));
        g_in{n, 1}=in;    
        g_mdls(n).i_mdl(1).K=K;
        % get ml regressor and latent variables (call probitem with weight decay penalty).
        [wn, mzn]=probitem(in, trg, vbprb.gamma, vbprb.acceps, vbprb.maxit, diag(explambda(ones(1, n_in))));
        g_mdls(n).i_mdl(1).beta=wn(:);
        g_mdls(n).i_mdl(1).P=diag(explambda(ones(1, n_in)));
        g_mdls(n).i_mdl(1).C=diag(1./explambda(ones(1, n_in)));
        g_mdls(n).i_mdl(1).n_in=n_in;    
        % store latent ml values
        allz(:,n,1)=mzn(:);
        n=n+1;
    end
    % we have now all distributions and the helper variables initialized and
    % are ready to store them into vbprb:
    vbprb.sys(sysno).alldim=alldim;
    vbprb.sys(sysno).g_mdls=g_mdls;
    vbprb.sys(sysno).latmeans=allz;
    g_vals(sysno).g_in=g_in;
    % now we know maxparlen and can thus initialize the posterior over H (diagonal Gamma). 
    vbprb.sys(sysno).g_p=g(ones(maxparlen,1));
    vbprb.sys(sysno).h_p=h(ones(maxparlen,1));
end
vbprb.allIgprobs=0.5*ones(2, nsubsets);
