function [KLsysind, P_sysind, KLsysmsr, P_sysmsr]=vbprb_getsysKL(skipcomp)
% function [KLsysind, P_sysind, KLsysmsr, P_sysmsr]=vbprb_getsysKL(vbprb, g_vals, skipcomp)
% is used to calculate the KL divergence between each systems posterior for
% the subset indicator variables and the overall posterior. 
% As such this KL divergence is a measure of "agreement" which can be used
% to display this additional information about these probabilities.
%
% IN
% vbprb :   object for variational Bayesian hierarchical probit GLM 
%           as generated from vballgprb and infered by vbprb_tran.
% g_vals:   additional information like targets and inputs that are used 
%           during inference.
% skipcomp: if true skip reference subset from the probability measures.
% 
% OUT
% KLsysind: KL divergences. [nsys x nsubsets]
% P_sysind: indicator probabilities expected from every system on it's own.
%           These probabilities follow from the approximated log marginal 
%           likelihood. Dimension: [n_I x nsubsets x nsys]
% KLsysmsr: KL divergences [nsys x 1] (only one distance of gene measures
%           per system)
% P_sysmsr: System specific gene measure: [nsys x nsubsets+1] (one more
%           than subsets for the empty subset)
%
% (C) P. Sykacek 2005.
% mod 2006: global vbprb g_vals allFI sysFI;
global vbprb;
global g_vals;
global allFI;
global sysFI;
refpos=vbprb.refsubspos;
get_FI();
% sysFI: [nsubsets x n_I x nsys]
% prior probabilities: (We use the expected prior here).
% vbprb.i_Postcnt: [n_I x 1] (posterior counts to get prior.
Prio=vbprb.i_Postcnt/sum(vbprb.i_Postcnt);
% posterior indicator probabilities from all sensors
% P_ind: [n_I x nsubsets].
P_ind=vbprb.allIgprobs;
if ~skipcomp
    nind=length(P_ind);
    % note: refpos was removed. the correct probability is thus at
    % refpos and we need to take the original guys starting at refpos...
    P_ind=[P_ind(:,[1:(refpos-1)]), [0.5; 0.5], P_ind(:,[refpos:nind])];
end
% now we need to obtain the indicator probabilities that result from all
% individual systems:
psi_vals=psi(vbprb.i_Postcnt)-psi(sum(vbprb.i_Postcnt));
% the calculation of sysFI requires us to add psi_vals accordingly:
nvals=length(psi_vals);
for n=1:nvals
    sysFI(:,n,:)=sysFI(:,n,:)+psi_vals(n);
end
% next we consider the reference subset in allFI and sysFI:
if ~skipcomp
    % see what we do above with P_ind: its the same here!
    %allFI=[allFI(:,[1:(refpos-1)]),log(0.5)*ones(nI,1), allFI(:,[refpos:nsubs])];
    % now we do the same for all sysFIs
    [nsubs, n_I, nsys]=size(sysFI);
    sysFI=[sysFI([1:(refpos-1)],:,:);log(0.5)*ones(1,n_I, nsys); sysFI([refpos:nsubs],:,:)];
end
% calculate P_sysind:
[nsubs, n_I, nsys]=size(sysFI);
P_sysind=zeros(n_I, nsubs, nsys);
allones=ones(1, nsubs);
for n=1:nsys
    logevdiff=[sysFI(:,2,n)-sysFI(:,1,n)]';
    P_sysind(1,:,n)=allones./(allones+exp(logevdiff));
    P_sysind(2,:,n)=allones-P_sysind(1,:,n);
end
% P_sysind: [n_I x nsubs x nsys]
% now we get the KL divergence:
KLsysind=zeros(nsys, nsubs);
for n=1:nsys
    [KLsysind(n,:)]=kldisc(P_ind', P_sysind(:,:,n)')';
end

% in preparation of geting the gene measure specific KL divergence between 
% the overall measure and each systems specific measure, we calculate the
% gene measure
doindP=0;
[P_msr]=vbprb_getmdlprbs(doindP, skipcomp);
% and all system specific gene measures including the empty gene. and the
% KL divergence between the overall measure and the system specific
% measure.
KLsysmsr=zeros(nsys, 1);
for n=1:nsys
    % new based on log evidences:
    allFI=sysFI(:,:,n)';
    nI=size(allFI,1);
    dFI=allFI(2,:)-allFI(1,:);
    P_thissysmsr=zeros(size(dFI));
    nsubs=length(P_thissysmsr);
    for m=1:nsubs
        P_thissysmsr(m)=1/(1+sum(exp(dFI([1:(m-1),(m+1):nsubs])-dFI(m))));
    end
    P_sysmsr(n,:)=P_thissysmsr;
    KLsysmsr(n)=kldisc(P_msr, P_sysmsr(n,:));
end